# kernel-joeghi

This is an experimental kernel for Mageia GNU/Linux with *low latency*
features.

As you might already know, over the years, a certain amount of
external patchsets were available for building Linux kernels with some
kind of "real-time" or "low-latency" features. This kernel tries to
incorporate some of them. Many projects of this kind already exist
over the net with the same aims, either as a kernel with patches
merged-in available in github (zen, pf-linux, linux-vd, etc.) or as an
extra kernel package built for a specific distribution.

Most of this job was inspired by the Thomas Backlund's "kernel-tmb"
RPM kernels, and Niccolò Costanza's "kernel-nrjQL" RPM kernels,
without whom this work wouldn't have been possible. To them goes my
gratitude.

Note this kernel differs from the current Mageia kernel-tmb's kernel
by the fact that kernel-joeghi uses the Mageia "kernel-desktop" as
starting point, then it tries to add further patchsets, some of which
were not included in kernel-tmb. The optimum goal achievement would be
to derive the SPEC file (as well the archive set) for this kernel almost
"automatically" from the main Mageia (desktop) kernel by using some
"magic" script. Note also that this kernel is not intended to be a
replacement for kernel-tmb.

Actually, with respect to stock mageia kernel-desktop, this kernel
mainly includes:

* Con Kolivas's MuQSS patchset for low latency scheduler. This is
  actually included in latest kernel-tmb **too**. See:

  * [http://ck.kolivas.org/patches/muqss/4.0/4.9](http://ck.kolivas.org/patches/muqss/4.0/4.9/)
  * [https://github.com/ckolivas/linux](https://github.com/ckolivas/linux/)

* TOI (Tux On Ice) for hybernation. See:

  * [https://github.com/NigelCunningham/tuxonice-kernel/tree/tuxonice-4.9](https://github.com/NigelCunningham/tuxonice-kernel/tree/tuxonice-4.9)

* Graysky2 for extra gcc optimization. See:

  * [https://github.com/graysky2/kernel_gcc_patch](https://github.com/graysky2/kernel_gcc_patch)

  as well as the Building Notes.

* NVME patchset for low power consumption of NVME disks when
  idle. See:

  * [https://github.com/damige/linux-nvme](https://github.com/damige/linux-nvme)

* Jans Axboe's Buffered WriteBack Throttling. See:

  * [http://brick.kernel.dk/snaps/wb-buf-throttle-v8.patch](http://brick.kernel.dk/snaps/wb-buf-throttle-v8.patch)

* The default TCP Traffic Congestion Control algorithm was changed from "cubic"
  to "bbr". Also other algorithms were made available. 

This kernel has the time frequency set to 100 Hz (default for
standard kernel-desktop is 1000 Hz), though it is *tickless*.


### Building Notes

Since it wouldn't be possible for the Building System to build a
kernel optimized for a specific CPU (``-march=native`` would result in
the CPU of the building system), the RPM Spec file has four new
conditional building flags that users might use for quickly rebuilding
the RPM package optimized for their CPU. While this provide tiny
optimization over the generic one, the Graysky2 benchmarks have
demonstrated the speed up, even very tiny, is not null. The extra
building flags are:

 * ``--with gcc_native``: optimizes for the native CPU

 * ``--with gcc_broadwell``: optimizes for the Intel CPU Broadwell

 * ``--with gcc_atom``: optimizes for the Intel CPU Atom. Atom
 has *in-order* pipelining while most of CPU supports 
 *out-of-order* pipelining, so worthwhile to compile for Atom with 
 this custom building flag.

 * ``--with gcc_harder_opt``: add extra ``-O3`` to the gcc optimization;
   this is usually not adviced for production as might produce a
   broken kernel, but you may try as an experiment (to me worked ;-)).

E.g. building the kernel with "``bm -l --with gcc_broadwell
kernel-joeghi.spec``", will rebuild the kernel optimized for the Broadwell
CPU. For example using "``bm -l --with gcc_atom --with gcc_harder_opt
kernel-joeghi.spec``" will rebuild the kernel optimized for CPU Atom (at
64bit) and will also push the extra ``-O3`` compiler
optimization. While "``bm -l --with gcc_native kernel-joeghi.spec``" will
auto-detect the CPU and rebuild the kernel optimized for that CPU.

Note that, according to the gcc man page, optimizing for Broadwell is
the highest Intel CPU supported by the Mageia's gcc 5.4. Newer CPUs
based on SkyLake or KabyLake will have to rely, at most, on the
Broadwell optimization, or would need a newer gcc version.

### Notes
For selecting an alternative **TCP traffic congestion control algorithm**,
you have to load the kernel module of the corresponding algorithm, e.g.

	modprobe tcp_yeah tcp_htcp 

then check for available algorithms, e.g.:

	sysctl net.ipv4.tcp_available_congestion_control

would return:

	net.ipv4.tcp_available_congestion_control = bbr reno cubic westwood veno vegas yeah htcp

then select the choosen algorithm, with:

	sysctl -w net.ipv4.tcp_congestion_control=yeah

Actually this kernel is provided and tested only for the *x86_64*
architecture. Note that exploring the kernel parameters to seek for
optimum values under certain circumstances is pretty hard and long,
and you achieve that only by testing, reading docs, bencmarking, and
mainly exchanging experiences each other. Of course all the possible
combinations of every single configurable build-time kernel
parameter could not be tested (at an hypotetical rate of 1 per
second that would probably exceed the age of the universe ;-)).



Enjoy.

###### Author: Giuseppe Ghibò <ghibo at mageia.org>
